#!/bin/sh
#####################################################################################################
# IBM Confidential
# Surveillance Insight for Financial Services: Install
# Copyright IBM Corp. 2017, 2018
# The source code for this program is not published or otherwise divested of its trade secrets,
# irrespective of what has been deposited with the U.S. Copyright Office.
######################################################################################################
SIFS_User_HomeDir=$1
echo SIFS_User_HomeDir=$SIFS_User_HomeDir
User=$2
echo User=$User
Group=$3
echo Group=$Group
Time_Period=$4
echo Time_Period=$Time_Period
Spark_Install_Location=$5
echo Spark_Install_Location=$Spark_Install_Location
HDFSFilePath=$6
echo HDFSFilePath=$HDFSFilePath
HDFSComplaintsPath=$7
echo HDFSComplaintsPath=$HDFSComplaintsPath
HDFSSampleDataFilePath=$8
echo HDFSSampleDataFilePath=$HDFSSampleDataFilePath
PYSPARK_PYTHON=$9
echo PYSPARK_PYTHON=$PYSPARK_PYTHON
Python3=${10}
echo Python3=$Python3
Pip3=${11}
echo Pip3=$Pip3
SparkSubmit=${12}
echo SparkSubmit=$SparkSubmit
script_Location=${13}
echo script_Location=$script_Location

echo "creating alias for trendanlysis"
echo "alias python3='$Python3'" >> ~/.bashrc
echo "alias pip3='$Pip3'" >> ~/.bashrc
echo "alias spark-submit='$SparkSubmit'" >> ~/.bashrc
#export PYSPARK_PYTHON=/usr/local/bin/python3.5
source ~/.bashrc

version=$(python3 -V 2>&1 | grep -Po '(?<=Python )(.+)')
echo "version="$version
if [[ -z "$version" ]]
then
    echo "No Python is not installed"
    exit 1 
fi

parsedVersion=$(echo "${version//./}")
if [[ "$parsedVersion" -ge "352" ]]
then 
    echo "Valid version"
else
    echo "Invalid Python Version, Valid Python Version is 3.5.2 or Greater"
  exit 1
fi

res=$(python3 -c 'import pkgutil; print(0 if pkgutil.find_loader("sklearn") else 1)')
if [ $res -eq 1 ]
then
	echo "sklearn Python Module is not installed"
	exit 1
fi

res=$(python3 -c 'import pkgutil; print(0 if pkgutil.find_loader("pandas") else 1)')
if [ $res -eq 1 ]
then
	echo "Pandas Python Module is not installed"
	exit 1
fi

res=$(python3 -c 'import pkgutil; print(0 if pkgutil.find_loader("numpy") else 1)')
if [ $res -eq 1 ]
then
	echo "numpy Python Module is not installed"
	exit 1
fi

res=$(python3 -c 'import pkgutil; print(0 if pkgutil.find_loader("statsmodels") else 1)')
if [ $res -eq 1 ]
then
	echo "statsmodels Python Module is not installed"
	exit 1
fi

res=$(python3 -c 'import pkgutil; print(0 if pkgutil.find_loader("gnip_trend_detection") else 1)')
if [ $res -eq 1 ]
then
	echo "gnip_trend_detection Python Module is not installed"
	exit 1
fi

res=$(python3 -c 'import pkgutil; print(0 if pkgutil.find_loader("requests") else 1)')
if [ $res -eq 1 ]
then
	echo "requests Python Module is not installed"
	exit 1
fi

echo "pre-checks Done!"

chown -R $User $Python3


echo "*************************************************"
echo "Invoking EntityExtraction and NLP"
echo "*************************************************"
echo ""


echo "Invoking PySpark Jobs for Complaints UseCase"
#sudo yum -y install dos2unix
cd $script_Location/scripts/
sudo dos2unix $script_Location/scripts/*.sh

sudo chmod -R 777 $script_Location/scripts/
sudo chown -R $User:$Group $script_Location/*


echo "Updating PySpark Jobs Script"
echo "$script_Location/ml.trend-detection/scripts/trend_detection.sh"
sudo sed -i "s#/home/sifsuser/spark-2.0.2-hadoop2.7/bin/spark-submit#$Spark_Install_Location/bin/spark-submit#g" $script_Location/scripts/trend_detection.sh
sudo sed -i "s#hdfs://localhost:8020/user/sifsuser/Complaints/output/complaints_features.csv#$HDFSFilePath$HDFSComplaintsPath$HDFSSampleDataFilePath#g" $script_Location/scripts/trend_detection.sh

#/bin/su -s /bin/bash -c  "$script_Location/scripts/trend_detection.sh $Time_Period" $User
/bin/su -s /bin/bash -c "export PYSPARK_PYTHON=/usr/local/bin/python3.5;export HADOOP_CONF_DIR=/usr/iop/4.2.0.0/hadoop/conf; echo 'alias python3="$Python3"' >> ~/.bashrc; echo 'alias pip3="$Pip3"' >> ~/.bashrc; echo 'alias spark-submit="$SparkSubmit"' >> ~/.bashrc; source ~/.bashrc; kinit -kt /etc/security/keytabs/sifsuser.keytab sifsuser@IBM.COM; $script_Location/scripts/trend_detection.sh $Time_Period;" $User
