# Trend Detection


## Installation  ##
Use the following command to install prerequisites for Python before installing it.  

    yum install gcc
    yum install sqlite-devel

### Download Python 3.5.2  
Download Python from the python official site:  

    cd /usr/src  
    wget https://www.python.org/ftp/python/3.5.2/Python-3.5.2.tgz  

Now extract the downloaded package.  

    tar xzf Python-3.5.2.tgz

### Compile Python Source 
Use below set of commands to compile python source code on your system using altinstall:  

    cd Python-3.5.2
    ./configure  
    make altinstall

make altinstall is used to prevent replacing the default python binary file /usr/bin/python.  
Now remove downloaded source archive file from your system:  


       rm Python-3.5.2.tgz
### Check Python Version 
Check the latest version installed of python using below command: 

    python3.5 -V

### Notes on parameters and alias

The following parameters are required for the script to run.
**These are created in the system automatically by a build script as they affect the whole environment. No action is required here, the values below are given for reference, checking and debugging etc.
    
	
	export PYSPARK_PYTHON=/usr/local/bin/python3.5
	export SPARK_HOME=/home/sifsuser/spark-2.1.1-bin-hadoop2.7
	
These alias can be added to sifsuser ~/.bashrc file:

 	alias python3='/usr/local/bin/python3.5'
	alias pip3='/usr/local/bin/pip3'
	alias spark-submit='/home/sifsuser/spark-2.1.1-hadoop2.7/bin/spark-submit'
After adding the above in the file. run the below command:

    source ~/.bash_aliases or source ~/.bashrc


### Install/upgrade pip, setuptools and wheel  
Each Python interpreter on your system needs its own install of pip, setuptools and wheel. The easiest way to install or upgrade these packages is by using the get-pip.py script.

    wget https://bootstrap.pypa.io/get-pip.py  
    python3 get-pip.py

## Install python modules



Install the following python libraries using pip3:

    pip3 install sklearn  
    pip3 install numpy    
    pip3 install pandas  
    pip3 install statsmodels  
    pip3 install gnip_trend_detection  
    pip3 install requests

**Check that packages are readable by other users than root and update permissions if required.** For example

    chmod -R 755 /usr/local/lib/python3.5/site-packages

- These modules must be available for **sifsuser**.   
- These modules are required on the **hadoop** **master**, but not anywhere else.

  
## Install Additional python modules

Trend-detection and on-demand trend-detection share the same library, therefore some additional modules need to be imported which are used solely by the on-demand trend-detection part. Install the following additional libraries using pip3:
    
    pip3 install ibm_db==2.0.8a 
    Pip3 install flask 
    pip3 install flask_cors
  


These modules must be available for sifsuser. 

If this causes permission errors as an alternative these libraries can be installed for sifsuser only: 
 
	sudo su sifsuser
	pip3 install ibm_db==2.0.8a --user
    Pip3 install flask --user
    pip3 install flask_cors --user



# Detecting Trends

A trend is a defined as change in the normal number of complaints we'd expect to see. Most of the time we are interested in finding sudden increases in the amounts of trends related to a particular theme. 

In the graphic below the number of complaints per day for one theme has been plotted for 30 days. The number of complaints starts increasing rapidly after June 4th. 


![Alt text](/docs/trend2.png)


The lighter blue line is the "raw" amount of complaints per day, and is very jagged - this is because the amount of complaints vary between weekdays and weekends. To make it easier to understand we also plot a smoothed line (the dark blue line) by taking the average over 7 days for each day. This removes the noise caused by weekdays/weekends.

The red dot indicates the day on which the trend started.

The script identifies all the combinations of dimensions (product, sub-product, issue, state, gender etc). The complaints are counted for each of these combinations, and evaluated to see if a trend can be found. 


# Running the program
Submit the program as a spark-submit job . Example: 

   
    spark-submit   
    --driver-class-path /home/sifsuser/lib/db2jcc4.jar   
    --conf="spark.driver.extraJavaOptions=-Djavax.net.ssl.trustStore=path_to_trust_store -Djavax.net.ssl.trustStorePassword=trust_store_password"   
    trendDetection.py -sd 2017-04-01 -ed 2017-06-30 -db2  

notes:    
- db2jcc4.jar is the  jdbc driver file  
- replace "path_to_trust_store" and "trust_store_password" with the actual parameters, as read from sifs.spark.properties  
- db2 parameter tells algorithm to read the db2 database  
- addtional python parameters for trend detection can also be set, see next section 

   
### Parameters:
These command line parameters are used when running the program:

<p><html></p>
<head>
      <title>HTML Table Header</title>
</head>
<body>
      <table border = "0">
         <tr>
            <th>parameter</th>
            <th>short form</th>
			<th>long form</th>
			<th>description</th>
			<th>example</th>
         </tr>
         <tr>
            <tr>
            <td>input file</td>
            <td>-i</td>
			<td>--input-file</td>
			<td>optional: file name of the input data file. Only if not reading data from db2</td>
			<td>example</td>
         </tr>
<tr>
            <tr>
            <td>start date</td>
            <td>-sd</td>
			<td>--start\_date</td>
			<td>start of the ranging over period</td>
			<td>2017-01-01</td>
         </tr>
<tr>
            <tr>
            <td>end date</td>
            <td>-ed</td>
			<td>--end\_date</td>
			<td>end date of the ranging over period</td>
			<td>2017-02-28</td>
         </tr>
<tr>
            <tr>
            <td>long window length</td>
            <td>-lw</td>
			<td>--long\_window\_length</td>
			<td>OPTIONALthe length in days of how far back to get the variability of a timeline </td>
			<td>default value if none given is 30</td>
         </tr>
<tr>
            <tr>
            <td>short window length</td>
            <td>-sw</td>
			<td>--short\_window\_length</td>
			<td>OPTIONAL the length in days of the last few days over which there might be a trend</td>
			<td>default value if none given is 7</td>
         </tr>
<tr>
            <tr>
            <td>config file</td>
            <td>-c</td>
			<td>--config\_file</td>
			<td>OPTIONAL the name of the config file</td>
			<td>config.cfg</td>
         </tr>

<tr>
            <tr>
            <td>verbose logging flag</td>
            <td>-v</td>
			<td>--verbose</td>
			<td>enable more discreptive logging</td>
			<td>-v</td>
         </tr>

<tr>
            <tr>
            <td>local filename</td>
            <td>-lo</td>
			<td>--local\_output\_file\_mode</td>
			<td>a local filename so that the trends  are saved to a local csv file not sent to db2 via the API ReST service. Files are saved in the \../output/ directory</td>
			<td>complaint_trends.csv</td>
         </tr>
<tr>
            <tr>
            <td>map zip to state</td>
            <td>-z</td>
			<td>--zipcodemap</td>
			<td>if the geo_ref field has the zip code but the state field is empty, this will map zip code to state</td>
			<td>-z</td>
         </tr>
</body>
</html>
<body> <table border = "1">
         
    
***

### Production Mode1 : data read from db2, results sent to UI
Submit the program as a spark job with the db2 driver parameter set. Example: 

    spark-submit --driver-class-path /home/sifsuser/lib/db2jcc4.jar --conf="spark.driver.extraJavaOptions=-Djavax.net.ssl.trustStore=<<path to trust store>> -Djavax.net.ssl.trustStorePassword=<<trust store password>>" trendDetection.py -sd 2017-04-01 -ed 2017-06-30 -db2  



### Production Mode2 : data read from hdfs, results sent to UI
The script can also read a data file stored on hdfs. The spark parameters for hdfs must be passed first (keytab and principal user) , then the python program name, and finally all the python program parameters. For example, this set of parameters will allow a hdfs file to be processed:  

    spark-submit --keytab /etc/security/keytabs/sifsuser.keytab --principal sifsuser@IBM.COM  trendDetection.py -i hdfs://<ip address>:8020/user/sifsuser/Complaints/output/one_category_trending_in_three_states.csv -v
 
important: manually copy the data file into the hdfs directory before running this script:


    hadoop fs -put /home/sifsuser/ml.trend-detection/data/one_category_trending_in_three_states.csv /user/sifsuser/complaints/output/complaints_test_data.csv/one_category_trending_in_three_states.csv



### Basic Testing mode: Data read from local csv file, results saved to local csv file 
1) local mode: trends are saved in a csv file in the local file structure. There is no connection with the UI . To run in local mode use the -lo <filename.csv> parameter, for example:

    python3 trendDetection.py  -i ../one_category_trending_in_three_states.csv -c config.cfg -v -lo output_test.csv -sd 2017-04-01 -ed 2017-06-30


### UI Testing mode: Data read from local csv file, results sent to UI  
2) UI mode: trends are sent to the UI by an API REST Service. To run in UI  mode **omit** the -lo <filename.csv> parameter
example:
`python3 trendDetection.py  -i ../data/one_category_trending_in_three_states.csv -c config.cfg -sd 2017-05-01 -ed 2017-06-30`  

Results should appear immediately in the UI. Details of the UI integration db/REST API service are not passed as parameters. These are defined in the global config file:  

    spark_sifs_config = '/home/sifsuser/lib/sifs.spark.properties'
    param = "ComplaintTrendsREST" 


This set set of parameters will work for a basic test of the script using the supplied test data and without using the spark functionality:

    -i ../data/one_category_trending_in_three_states.csv
    -c ../config/config.cfg  
    -v  
    -sd 2017-04-01  
    -ed 2017-06-30
    -lo test_output.csv
For example, this set of parameters will allow a local file to be processed and the trend sent to a local output file:  

    python3 trendDetection.py  -i ../one_category_trending_in_three_states.csv -c config.cfg -v -lo output_test.csv -sd 2017-04-01 -ed 2017-06-30 



# File structure
The code is shipped with the following file structure:

|  
|-config   	 directory for configuration  
|-data  	 directory for data  
|-docs       directory for documentation  
|-results    directory for any local output  
|-scripts    directory for test scripts    
|-src        directory for source code  
|-test       directory for additional test materials  
README file



# Global Configuration parameters

The url for the REST service to update the database is stored in the **'/home/sifsuser/lib/sifs.spark.properties'** file. The program reads parameter:

    "ComplaintTrendsREST"='https://<ip address>:<port>/complaintsservices/surveillance/v1/complaint/createTrend'

where the "ip address" and "port" parameters should be replaced with the real values for the system. 

# Complaints Configuration parameters

All other configuration details are stored in a config file. The 
 config directory is always called "config" and the name of the specific config file is passed in a parameter -c. If no config file is given the default "config.cfg" will be used. 

**Fields list:**   
this is the list of all the columns in the data file over which complaints will be aggregated ( ie counted) to see if there are any trends. The field names should be added exactly as they appear in the data file separated by a commas without spaces. For example:  
	 THEME,SUB_THEME,PRODUCT,GEO  


    fields_list = THEME,SUB_THEME,PRODUCT,GEO


**Poisson model params**

    model_config = dict()
    model_name = 'Poisson'
    model_config['mode'] = 'lc'
    model_config['period_list'] = 'day'
    model_config['alpha'] = '0.99'
    poisson_threshold = 0.35

**Mann kendall trend detection**

    mk_confidence = 0.05

Eliminate 'candidate' combinations of timelines that dont meet basic requirements

    min_records = 10  # ie minimum total count over the window
    min_per_day = 5  # ie minimum count on any one day

pre-filter for a given timeline of complaint counts:

    pre_filter_critiera = dict()
    pre_filter_critiera['pf_long_window_min'] = 10   # 1)  there must be at least 10 complaints in the rolling window
    pre_filter_critiera['pf_short_window_min'] = 10  # 2) there should be at least 10 complaints in the trend window
    pre_filter_critiera['pf_mean_ratio'] = 1.0  # 3) the average short window should be greater than avg in long window




