# On Demand Trend Detection


## Overview
The on-demand trend algorithm integrates with the complaints User-Interface explore page. 
On that page a user can choose a number of features of interest: Category, Process, Product, Age and Geography and their corresponding values.

<img src="../docs/explore.jpg" alt="screenshot of explore screen">

For example the user may select these values:  

    {"THEME": ["Complaint_Delay"], "CUSTOMER_AGE": ["30-40", "40-50"], "GEO": ["TX","FL"]}}
    
This is submitted to the on-demand API which determines the trend chart for this combination. 
Behind the scenes the on-demand trend-detection algorithm will derive a time-series of counts of complaints per day for the date range given. For example:


    {"description": "CUSTOMER_AGE:30-40&THEME:Complaint_SalesPractice", "complaintcount": {"trendinglast30days": "+0.0%", "30dayavg": 0.0, "7dayavg": 0.0, "earlytrend": null, "trendinglast7days": "+0.0%", "dates": [{"date": "2018-03-08", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-09", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-10", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-11", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-12", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-13", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-14", "smoothcount": 0.0, "actualcount": 0}]}, "product": "all", "riskscore": 0.0, "timeperiod": 7, "theme": "Complaint_SalesPractice", "trendtypeid": 3, "geo": "all", "trendattributes": [{"MAX_AGE": 40}, {"THEME": "Complaint_SalesPractice"}, {"MIN_AGE": 30}], "trendstartdate": null}

**Note: on-demand trend detection does not work for key words.**

## Json Format

### input json
The algorithm is expecting a request with one parameter called **query**:

    {"query" : {"THEME": ["Complaint_Delay"], "CUSTOMER_AGE": ["30-40", "40-50"], "GEO": ["TX","FL"]}}}
or  

    {"query" : {"THEME": ["Complaint_Delay"], "CUSTOMER_AGE": ["30-40", "40-50"], "GEO": ["TX","FL"], "STARTDATE": "2017-04-01", "ENDDATE":"2017-05-01"}}
    


    
Notes:


1. All field names must align with the column names in db2 table complaint_features

1. All field values must be in a list or array, even if only one value is sent. For example: 

  
    valid		"CUSTOMER\_AGE": ["30-40"]  
	valid 		"CUSTOMER\_AGE": ["30-40", "40-50"]  
	not valid	"CUSTOMER\_AGE": "30-40"  



1. The trend detection is run for 4 different time frames: from today looking back 7, 30, 90 and 365 days. These correspond to the timeframe drop-down options in the trend chart page. This will produce 4 different rows in the output json.


1. The start date and end date params are optional. If the ENDDATE  and STARTDATE paramaters are set in the query, then trends will be detected over this range.  
2. If no date range is given the algorithm will default in the end date as todays date, and then calculate 7,30, 90 and 365 days prior etc. 
3. the range of calculation periods is set in the config.cfg file.

### output json
the output will have one parameter called trends with one or more rows of trend time series:


     {"trends":{"description": "CUSTOMER_AGE:30-40&THEME:Complaint_SalesPractice", "complaintcount": {"trendinglast30days": "+0.0%", "30dayavg": 0.0, "7dayavg": 0.0, "earlytrend": null, "trendinglast7days": "+0.0%", "dates": [{"date": "2018-03-08", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-09", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-10", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-11", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-12", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-13", "smoothcount": 0.0, "actualcount": 0}, {"date": "2018-03-14", "smoothcount": 0.0, "actualcount": 0}]}, "product": "all", "riskscore": 0.0, "timeperiod": 7, "theme": "Complaint_SalesPractice", "trendtypeid": 3, "geo": "all", "trendattributes": [{"MAX_AGE": 40}, {"THEME": "Complaint_SalesPractice"}, {"MIN_AGE": 30}], "trendstartdate": null}}
    
## Installation  ##

On Demand Trend Detection should be installed after Trend Detection as it shares most of the same python modules. This document assumes that that Trend Detection has been installed already and is working. The installation instructions give the **additional steps only** to get on-demand trend detection working



## Install Additional python modules

Install the following additional libraries using pip3:

    
    Pip3 install flask 
    pip3 install flask_cors
  
**Check that packages are readable by other users than root and update permissions if required.** For example

    chmod -R 755 /usr/local/lib/python3.5/site-packages 

The following module must be installed as **sifsuser**:


	sudo su sifsuser
	pip3 install ibm_db==2.0.8a --user
  

# Public Certificate for Secure Connection to db2

As part of the configuration of a secure db2 instance and securing communications for IBM Streams, a public certificate was generated and imported into the IBM Streams keystore. In a default installation this certificate is stored as: 
**/home/db2inst1/SIDB2.arm**

 For example see these links:  
<https://www.ibm.com/support/knowledgecenter/SSWTQQ_2.0.2/install/t_trd_secureibmstreams.html?view=embed> 
<https://www.ibm.com/support/knowledgecenter/SSWTQQ_2.0.2/install/t_trd_secureibmstreams.html?view=embed> 


    Extract the certificate that you created to a file. The file can be distributed to 
	computers that run clients that use SSL communications with the DB2 server. For example, you can use gsk8capicmd 
	to extract the certificate to a file that is named SIDB2.arm:
	 ./gsk8capicmd_64 -cert -extract -db /home/db2inst1/SIdb2keys.p12 -pw YourPassword -label "db2-selfsigned"
	 -target "/home/db2inst1/SIDB2.arm" -format ascii -fips
    
 
This certificate and location must be made available to the on-demand trend algorithm via a paramater in the on-demand REST API config file:   
 **./config/REST.cfg**		( see below for more details on this file.)

The following entry should be made in this config file:

    [server]
    ...
    sslServerCert = /home/db2inst1/SIDB2.arm


note: for testing locally, on windows etc the certificate can be copied to the local machine and then accessed using parameter **-c** when the program is run:

    python3 onDemandTrendDetectionRestAPI.py -c ./path to local certificate.arm
    

# Running the program

To run the program with the default set of options:

    cd /home/sifsuser/complaints/trend-detection/src
    python3 onDemandTrendDetectionRestAPI.py
    

The following optional parameters can also be used:



### Parameters:
These command line parameters are used when running the program:

<p><html></p>
<head>
      <title>HTML Table Header</title>
</head>
<body>
      <table border = "0">
         <tr>
            <th>parameter</th>
            <th>short form</th>
			<th>long form</th>
			<th>description</th>
			<th>example</th>
         </tr>
 <tr>
            <tr>
            <td>global SIFS config file</td>
            <td>-gc</td>
			<td>--global-sifs-config-file</td>
			<td>Path to the GLobal SIFS config file. If no entry the default '/home/sifsuser/lib/sifs.spark.properties is used. Use this for testing on Windows environments.'
			<td>-gc ./config/sifs.spark.properties'</td>
         </tr>
         <tr>
            <tr>
            <td>REST API config file</td>
            <td>-rc</td>
			<td>--rest-config-file</td>
			<td>Path to the REST API Config file. If no entry the default ./config/REST.cfg is used
			<td>-rc ./config/new_REST.cfg</td>
         </tr>

<tr>
            <tr>
            <td>trends config file</td>
            <td>-tc</td>
			<td>--trends-config-file</td>
			<td>Path to the config file for trends detection algorithm. If no entry, the default is '../config/config.cfg'</td>
			<td>-tc ../config/new\_config.cfg</td>
         </tr>
<tr>
            <tr>
            <td>log file path</td>
            <td>-lg</td>
			<td>--log-file</td>
			<td>path to the log file. <br />If no entry the default is '../config/logs/on\_demand\_trends.log'</td>
			<td>-lg ../config/logs/new\_on\_demand\_trends.log</td>
         </tr>

<tr>
            <tr>
            <td>certificate file path</td>
            <td>-c</td>
			<td>--cert-file</td>
			<td>path to the db2 ssl public certificate.<br /> If no entry the default is read from the REST API config file</td>
			<td>-c ./cert.arm</td>
         </tr>
<tr>
            <tr>
            <td>no ssl verification</td>
            <td>-nv</td>
			<td>--no\_ssl_verification</td>
			<td>use this to switch off ssl certificate verification for testing only</td>
			<td>-nv</td>
         </tr>


</body>
</html>
<body> <table border = "1">
         
# Linking with the Explore Screen    

The UI explore screen dynamically creates a query which is used by the on-demand trend detection algorithm to produce the trend chart. This section explains how the UI maps to the query and the chart produced.   

<img src="../docs/explore.jpg" alt="screenshot of explore screen">
  
<table>
<thead>
<tr>
  <th>Explore Screen		</th>
  <th>Mapping to db2 column in SIFS.COMPLAINT_FEATURES</th>
  <th>Example</th>
</tr>
</thead>
<tbody>
<tr>
  <td>Categories </td>
  <td>THEME </td>
  <td>“THEME”: [“Complaint_SalesPractice”] </td>
</tr>

<tr>
  <td>Product </td>
  <td>PRODUCT </td>
 <td>“PRODUCT”: [“Mortgage”] </td>
</tr>

<tr>
  <td>Process </td>
  <td>SUB_THEME </td>
 <td>“SUB_THEME”: [“Process_Appraisal”] </td>
</tr>

<tr>
  <td>Age </td>
  <td>CUSTOMER_AGE </td>
 <td>“CUSTOMER_AGE”: [“30-40”] </td>
</tr>
<tr>
  <td>Geography </td>
  <td>GEO </td>
 <td> “GEO”: [“CA”, “NY”] </td>
</tr>
<tr>
  <td>Keywords </td>
  <td>No mapping: 
on-demand query doesn’t include keyword searches </td>
 <td>N/A </td>
</tr>
<tr>
  <td>Not on screen </td>
  <td>STARTDATE </td>
 <td>“STARTDATE”: “2017-05-01” </td>
</tr>
<tr>
  <td>Not on screen </td>
  <td>ENDDATE </td>
 <td>“ENDDATE”: “2017-06-12” </td>
</tr>

</tbody>
</table>

**Creating the query:  **


1. Top level name = “query”: {“query”: {…selection fields ...}} 
2. All selection fields (apart from dates) are in an array, even if only one value is selected as in example column in tables above. Examples:  
-"THEME”: [“Complaint_SalesPractice”]  
-“GEO”: [“CA”,”FL”] `  
  


1. Dates are OPTIONAL:     
Send no startdate, end date parameter: 
a.	enddate is set to today, 
b.	startdate is set by date difference in days in configuration parameter time_periods: time_periods = [365]  



1. Send the start date and enddate parameter. I’m assuming that in the future explicit start and end date controls will be requested for this screen, in which case they can be used to set startdate and enddate directly  
a.	“STARTDATE”: “2017-05-01”  
b.	“ENDDATE”: “2017-06-12”  


# Testing

The REST API for on-demand trends can be tested using the curl command:

1) Create a text file called query.json with the following entry:


    {"query" : {"THEME": ["Complaint_Delay"], "CUSTOMER_AGE": ["30-40", "40-50"], "GEO": ["TX","FL"]}}

in the same directory run the following cURL command - update  the ip address, port and @filename.json as appropriate:  

    curl -k -H 'Content-Type: application/json' -X POST --data-binary @query.json https://ip_address:port/analytics/models/v1/on_demand_trend/
    
The expected response is a json structure giving the complaint counts per day and other trend statistics:


1) Create a text file called query2.json with the following entry:

    
    {"query" : {"THEME": ["Complaint_Delay"], "CUSTOMER_AGE": ["30-40", "40-50"], "GEO": ["TX","FL"], "STARTDATE": "2017-04-01", "ENDDATE":"2017-05-01"}}
    

in the same directory run the following cURL command - update  the ip address, port and @filename.json as appropriate:  

    curl -k -H 'Content-Type: application/json' -X POST --data-binary @query.json https://ip_address:port/analytics/models/v1/on_demand_trend/
    
The expected response is a json structure giving the complaint counts per day and other trend statistics:


# On-Demand Complaints Configuration parameters

Configuration parameters are stored in 3 different files:

1. Global Configuration parameters
1. On-Demand Trend detection REST_API parameters
1. Trend Detection algorithm parameters


## 1) Global Configuration parameters

The url and connection details for the db2 database instance is stored in the global SIFS config file:  '/home/sifsuser/lib/sifs.spark.properties' . The command line parameter **-gc** can be used for testing or debugging to override this.   
These values are used:

    # JDBC related properties
    db2jdbcurl=jdbc:db2://ip:port/SIFS:sslConnection=true;currentSchema=SIFS;
    db2user=db2user
    db2password=db2password

## 2) On-Demand Trend detection REST_API parameters

Parameters used for configuring the on-demand REST API and logging are stored in a separate file. If the REST_API is run without a xx paramater then a default file ./config/REST.cfg is read:  

    [server]
    Port = 5002
    Debug = False
    certPath = security/server.crt
    keyPath = security/server.key
    sslServerCert = /home/db2inst1/SIDB2.arm
    
    
    [log_file]
    LogFileName = logs/on_demand_trends

Either update these values or use a different REST API config file accessed by the **-rc** parameter.

## 3) Trend Detection algorithm parameters

All other parameters for the trend detection algorithm are stored in the main config file ./config/config.cfg. This can be changed with the **-tc** parameter. The main( ie batch) and the on-demand trend detection algorithm share these parameters so that the trends are detected in a consistent way. 
**See the main trend-detection readme for details of these parameters.** 

These are the parameters in the file that are used only for the on-demand timelines

Long Window  - used to establish the baseline for a trend , basically the average count over this period. example:    
**long_window = 30**

short window - this is the length of the shortest moving timeframe used to identify if a trend is occurring. A trend is said to exist when most of the counts increase compared to the previous days value over the short window. example:  
**short_window = 7**

number of days for averaging the raw complaint count. e.g. use 7 day average to remove weekly seasonality. Counts of complaints vary hugely between weekends and weekdays, which makes detecting trends difficult. By averaging over 7 days, this weekly seasonality is reduced.   
**Note 7 days is the optimal smoothing factor,** if fewer days are used then weekend/weekday variability is increased. if more than 7  is used then extra lagging/smoothing makes trend detection more difficult.  
**smoothfactor = 7** 

DAYS\_BETWEEN\_TRENDS: trend on days that are less than this factor apart will be joined up to find the earliest date the trend started. Example:    
**DAYS\_BETWEEN_TRENDS = 1**

number of time periods to calculate the on-demand timelines. 
 give a  list in python format e.g.If we want one on-demand timeperiod looking back in time one year then set this to [365]. example:  
 **time_periods = [365]**


